import sys

sys.path.append('..')

import digitalocean
from digitalocean import DataReadError, TokenError

from dobg.helper.confighandler import ConfigHandler
from dobg.exceptions.configexceptions import InvalidTokenException


def list_images(args):
    """ Lists all of Droplets """

    token = ConfigHandler.get_config_setting('token')

    manager = digitalocean.Manager(token=token)

    try:
        images = manager.get_all_images()
    except (DataReadError, TokenError):
        raise InvalidTokenException

    result = '{:10}{:46}{:15}{:37}{}\n'.format('Id:', 'Name:', 'Distribution:', 'slug:', 'Regions:\n')
    for image in images:
        result += '{:<10}{:46}{:15}{:37}{}\n'.format(image.id, image.name, image.distribution, image.slug if image.slug else 'N/A', ', '.join(image.regions))

    print(result)

        
        
    
    


import sys

sys.path.append('..')

import argparse

from .commands.settoken import set_token
from .commands.createdroplet import create_droplet
from .commands.deletedroplet import delete_droplet
from .commands.listdroplets import list_droplets
from .commands.listsizes import list_sizes
from .commands.listimages import list_images

from .helper.confighandler import ConfigHandler
from .helper.exceptionhandler import ExceptionHandler

def main():

    # main parser
    parser = argparse.ArgumentParser(prog="dobg", description="", epilog="Use <command> --help to see how to use certain command.")

    # subparsers for commands
    commands = parser.add_subparsers(title="commands", metavar='')

    # createdroplet command
    createdroplet_command = commands.add_parser('createdroplet', help="Creates a Droplet")
    createdroplet_command.add_argument("name", help="Name of a droplet")
    createdroplet_command.add_argument("-r", "--region", default="fra1", help="The unique slug identifier for the region that you wish to deploy in.")
    createdroplet_command.add_argument("-i", "--image", default="ubuntu-16-04-x64", help="Unique slug identifier for public image")
    createdroplet_command.add_argument("-s", "--size", default="s-1vcpu-1gb", help="The unique slug identifier for the size that you wish to select for this Droplet.")
    createdroplet_command.set_defaults(func=create_droplet)

    # deletedroplet command
    deletedroplet_command = commands.add_parser('deletedroplet', help="Destroys a Droplet")
    deletedroplet_command.add_argument("id", type=int, help="Id that uniquely identifies a Droplet.")
    deletedroplet_command.set_defaults(func=delete_droplet)

    # listdroplets command
    listdroplets_command = commands.add_parser('listdroplets', help="Lists all Droplets")
    listdroplets_command.set_defaults(func=list_droplets)

    # listsizes command
    listsizes_command = commands.add_parser('listsizes', help="Lists all Sizes available")
    listsizes_command.set_defaults(func=list_sizes)

    # listimages command
    listimages_command = commands.add_parser('listimages', help="Lists all Images available")
    listimages_command.set_defaults(func=list_images)

    # settoken command
    settoken_command = commands.add_parser('settoken', help="Sets the authentication token in config file")
    settoken_command.add_argument('token', help="Authentication token provided by Digital Ocean")
    settoken_command.set_defaults(func=set_token)

    args = parser.parse_args()

    if len(args.__dict__) < 1:
        parser.print_help()
        parser.exit()

    try:
        args.func(args)
    except Exception as e:
        exception_handler = ExceptionHandler(e)
        exception_handler.handle()


import sys

sys.path.append('..')

import digitalocean
from digitalocean import DataReadError, TokenError

from dobg.helper.confighandler import ConfigHandler
from dobg.exceptions.configexceptions import InvalidTokenException


def list_sizes(args):
    """ Lists all of Droplets """

    token = ConfigHandler.get_config_setting('token')

    manager = digitalocean.Manager(token=token)

    try:
        sizes = manager.get_all_sizes()
    except (DataReadError, TokenError):
        raise InvalidTokenException

    result = '{:18}{:<10}{:<8}{:<8}{:<20}{}\n'.format('Slug:', 'Memory:', 'CPUs:', 'Disk:', 'Price per month:', 'Regions:\n')
    for size in sizes:
        result += '{:18}{:<10}{:<8}{:<8}{:<20}{}\n'.format(size.slug, size.memory, size.vcpus, size.disk, size.price_monthly, ', '.join(size.regions))

    print(result)

        
        
    
    


import sys

sys.path.append('..')

import digitalocean

from dobg.helper.confighandler import ConfigHandler


def create_droplet(args):
    """ Creates Droplet """    
    
    token = ConfigHandler.get_config_setting('token')
    
    droplet = digitalocean.Droplet(token=token,
                                    name=args.name,
                                    region=args.region,
                                    image=args.image,
                                    size_slug=args.size,
                                    backups=True)

    droplet.create()

    print("You have successfuly created Droplet!\
        \nId: {}, Name: {}, Size: {}, Image: {}, Region: {}".format(droplet.id, droplet.name, droplet.size_slug, 
                                                                                        droplet.image, droplet.region))
    
    


import sys

sys.path.append('..')

import digitalocean
from digitalocean import DataReadError, TokenError

from dobg.helper.confighandler import ConfigHandler
from dobg.exceptions.configexceptions import InvalidTokenException


def list_droplets(args):
    """ Lists all of Droplets """

    token = ConfigHandler.get_config_setting('token')

    manager = digitalocean.Manager(token=token)
    
    try:
        droplets = manager.get_all_droplets()
    except (DataReadError, TokenError):
        raise InvalidTokenException

    if len(droplets) == 0:
        print('No droplets.')
        return

    # make a helper function that will accept droplet and format print like this
    result = '{:15}{:40}{:10}{:18}{}'.format('ID:', 'Name:', 'Region:', 'Size:', 'Image:\n')
    for droplet in droplets:
        result += '{:<15}{:40}{:10}{:18}{}\n'.format(droplet.id, droplet.name, droplet.region['slug'], droplet.size_slug, droplet.image['slug'])

    print(result)

        
        
    
    


import json
import os
import sys

sys.path.append('..')

from dobg.exceptions.configexceptions import MissingTokenException


# Helper class for config file
class ConfigHandler:
    config_file = os.path.dirname(os.path.dirname(os.path.realpath(__file__))) + '/dobg-config.json'

    @classmethod
    def get_config_setting(cls, key):
        """ Gets property from config dictionary"""

        config_data = cls.get_config_data()
        
        # check if authentication token exists
        if key == 'token':
            token = config_data.get(key, False)
            if token == False:
                raise MissingTokenException

        return config_data.get(key, False)

    @classmethod
    def set_config_setting(cls, key, value):
        """ Sets property in config dictionary """

        config_data = cls.get_config_data()
        config_data[key] = value
        if cls.set_config_data(config_data):
            print("Changes made in config file:\n\n{}: {}\n".format(key,value))

    @classmethod
    def get_config_data(cls):
        """ Opens config file, reading and returning data from it as a dictionary """

        mode = 'r' if os.path.exists(cls.config_file) else 'w+'
        try:
            with open(cls.config_file, mode) as json_file:

                if mode == 'w+':
                    data = {}
                    json.dump(data, json_file, ensure_ascii=False, indent=4)
                    json_file.seek(0)

                return json.load(json_file)
        except: 
            print("Error: could not open config file for reading!")
            sys.exit()


    @classmethod
    def set_config_data(cls, data):
        """ Opens config file and rewriting it with data dictionary dumped as json """

        try:
            with open(cls.config_file, 'w') as json_file:

                json.dump(data, json_file, ensure_ascii=False, indent=4)
                return True
        except:
            print("Error: could not open config file for writing!")
            sys.exit()


import sys

sys.path.append('..')

from dobg.helper.confighandler import ConfigHandler


def set_token(args):
    """ Sets a Digital Ocean authentication token in configuration file """

    answer = input('Are you sure you want to set this token? (yes/no)\n\nToken: {}\n\n'.format(args.token))
    print()
    
    if answer in ('y', 'Y', 'yes', 'YES', 'Yes'):
        ConfigHandler.set_config_setting('token', args.token)



class TokenException(Exception):
    """ """


class InvalidTokenException(TokenException):

    def __str__(self):
        return "Your Digital Ocean authentication token is invalid.\
                    \nPlease, set valid token with 'settoken' command."


class MissingTokenException(TokenException):

    def __str__(self):
        return "It seems like you haven't set up your Digital Ocean authentication token. \
                    \nPlease, set your token using 'settoken' command."


import sys

sys.path.append('..')

from digitalocean import DataReadError

# must import full path to the exception
from dobg.exceptions.dropletexceptions import InvalidIdException 
from dobg.exceptions.configexceptions import TokenException


class ExceptionHandler:
    
    def __init__(self, exception):
        self.exception = exception

    
    # this function for now does the same job for every kind of exception (only prints it's message).
    def handle(self):
        """ Handles thrown exceptions from main script """

        if isinstance(self.exception, TokenException):
            print(self.exception)

        elif isinstance(self.exception, InvalidIdException):
            print(self.exception)
        
        else:
            print(self.exception)
        


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="dobg",
    install_requires=['requests', 'python-digitalocean>=1.14.0'],
    version="0.0.22",
    entry_points = {
        "console_scripts": ['dobg = dobg.dobg:main']
        },
    author="Marko Latinovic",
    author_email="markolat@outlook.com",
    description="Digital Ocean CLI tool for managing Droplets",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/markolat/dobgcli",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: Freely Distributable",
        "Operating System :: OS Independent",
    ],
)


import sys

sys.path.append('..')

import digitalocean
from digitalocean import DataReadError, TokenError

from dobg.helper.confighandler import ConfigHandler
from dobg.exceptions.configexceptions import InvalidTokenException
from dobg.exceptions.dropletexceptions import InvalidIdException


def delete_droplet(args):
    """ Destroys Droplet """
     
    token = ConfigHandler.get_config_setting('token')

    manager = digitalocean.Manager(token=token)

    try:
        droplet = manager.get_droplet(int(args.id))
    except (DataReadError, TokenError):
        raise InvalidTokenException
    except Exception: # cannot import NotFoundError here
        raise InvalidIdException

    droplet.destroy()
    print("You have successfuly destroyed Droplet!\
        \nId: {}, Name: {}, Size: {}, Image: {}, Region: {}".format(droplet.id, droplet.name, droplet.size_slug, 
                                                                                        droplet.image['slug'], droplet.region['slug']))
    
    